<?php

namespace app\admin\service\goods;

use app\model\Goods;
use app\model\GoodsComponent;
use app\model\GoodsDetail;
use app\model\GoodsLabel;
use app\model\GoodsRunning;
use app\model\GoodsSku;
use app\model\SysCategory;
use app\model\SysSerial;
use support\Db;
use support\exception\BusinessException;
use support\Redis;
use support\Request;
use support\Response;

class GoodsService
{
    public static function selectAll($goodsIds)
    {
        $goods = Goods::where('goods_status', 'ON')
            ->when($goodsIds != '', function ($query) use ($goodsIds) {
                $query->whereIn('goods_id', $goodsIds);
            })
            ->select('goods_id', 'goods_name')
            ->get();

        return json_success('', $goods);
    }

    public static function select(Request $request, $classify = "GOODS")
    {
        $page = $request->get('page');
        $pageSize = $request->get('pageSize');
        $goodsName = $request->get('goods_name', '');
        $categoryId = $request->get('join_goods_category_id', null);

        $rows = Goods::with([
            'category' => function ($query) {
                $query->select('category_id', 'category_name');
            },
            'running' => function ($query) {
                $query->select('join_running_goods_id', 'goods_running_storage');
            },
            'supplier' => function ($query) {
                $query->select('supplier_id', 'supplier_name');
            }
        ])->select('goods_id', 'join_goods_category_id', 'join_goods_supplier_id', 'goods_status', 'goods_sales_price', 'goods_category', 'goods_name', 'goods_title', 'goods_cover', 'goods_sort', 'goods_addtimes')
            ->when($goodsName != '', function ($query) use ($goodsName) {
                $query->where(function ($q) use ($goodsName) {
                    $q->where('goods_name', 'like', '%' . $goodsName . '%')
                        ->OrWhere('goods_title', 'like', '%' . $goodsName . '%');
                });
            })->when($categoryId != null, function ($query) use ($categoryId) {
                $query->where('join_goods_category_id', $categoryId);
            })
            ->when($classify != '', function ($query) use ($classify) {
                if ($classify == 'GOODS') {
                    $query->whereIn('join_goods_category_id', ['6', '7', '8', '9', '10', '11', '12', '30']);
                } else {
                    $query->where('goods_classify', $classify);
                }
            })
            ->orderBy('goods_addtimes', 'DESC')
            ->forPage($page, $pageSize)
            ->get()
            ->toArray();

        $total = Goods::when($goodsName != '', function ($query) use ($goodsName) {
            $query->where(function ($q) use ($goodsName) {
                $q->where('goods_name', 'like', '%' . $goodsName . '%')
                    ->OrWhere('goods_title', 'like', '%' . $goodsName . '%');
            });
        })->when($categoryId != null, function ($query) use ($categoryId) {
            $query->where('join_goods_category_id', $categoryId);
        })->when($classify != '', function ($query) use ($classify) {
            if ($classify == 'GOODS') {
                $query->whereIn('join_goods_category_id', ['6', '7', '8', '9', '10', '11', '12', '30']);
            } else {
                $query->where('goods_classify', $classify);
            }
        })
            ->count();

        foreach ($rows as &$row) {
            $row['goods_cover'] = getenv('STORAGE_DOMAIN') . $row['goods_cover'];
            if (isset($row['running'])) {
                $row['running']['goods_running_storage'] = intval($row['running']['goods_running_storage']);
            }
        }

        return json_success('', compact('rows', 'page', 'pageSize', 'total'));
    }

    public static function selectSpecial(Request $request)
    {
        $page = $request->get('page');
        $pageSize = $request->get('pageSize');
        $goodsName = $request->get('goods_name', '');
        $categoryId = $request->get('join_goods_category_id', null);
        if ($categoryId == null) {
            $categoryId = [65, 43];
        } elseif (is_string($categoryId)) {
            $categoryId = [$categoryId];
        }

        $rows = Goods::with([
            'category' => function ($query) {
                $query->select('category_id', 'category_name');
            },
            'running' => function ($query) {
                $query->select('join_running_goods_id', 'goods_running_storage');
            },
            'supplier' => function ($query) {
                $query->select('supplier_id', 'supplier_name');
            }
        ])->select('goods_id', 'join_goods_category_id', 'join_goods_supplier_id', 'goods_status', 'goods_sales_price', 'goods_category', 'goods_name', 'goods_title', 'goods_cover', 'goods_sort', 'goods_addtimes')
            ->when($goodsName != '', function ($query) use ($goodsName) {
                $query->where(function ($q) use ($goodsName) {
                    $q->where('goods_name', 'like', '%' . $goodsName . '%')
                        ->OrWhere('goods_title', 'like', '%' . $goodsName . '%');
                });
            })->whereIn('join_goods_category_id', $categoryId)
            ->orderBy('goods_addtimes', 'DESC')
            ->forPage($page, $pageSize)
            ->get()
            ->toArray();
        $total = Goods::when($goodsName != '', function ($query) use ($goodsName) {
            $query->where(function ($q) use ($goodsName) {
                $q->where('goods_name', 'like', '%' . $goodsName . '%')
                    ->OrWhere('goods_title', 'like', '%' . $goodsName . '%');
            });
        })->whereIn('join_goods_category_id', $categoryId)->count();

        foreach ($rows as &$row) {
            $row['goods_cover'] = getenv('STORAGE_DOMAIN') . $row['goods_cover'];
            if (isset($row['running'])) {
                $row['running']['goods_running_storage'] = intval($row['running']['goods_running_storage']);
            }
        }

        return json_success('', compact('rows', 'page', 'pageSize', 'total'));
    }

    public static function selectPicking(Request $request)
    {
        $page = $request->get('page');
        $pageSize = $request->get('pageSize');
        $goodsName = $request->get('goods_name', '');

        $rows = Goods::with([
            'category' => function ($query) {
                $query->select('category_id', 'category_name');
            },
            'running' => function ($query) {
                $query->select('join_running_goods_id', 'goods_running_storage');
            },
            'supplier' => function ($query) {
                $query->select('supplier_id', 'supplier_name');
            }
        ])->select('goods_id', 'join_goods_category_id', 'join_goods_supplier_id', 'goods_status', 'goods_sales_price', 'goods_category', 'goods_name', 'goods_title', 'goods_cover', 'goods_sort', 'goods_addtimes')
            ->when($goodsName != '', function ($query) use ($goodsName) {
                $query->where(function ($q) use ($goodsName) {
                    $q->where('goods_name', 'like', '%' . $goodsName . '%')
                        ->OrWhere('goods_title', 'like', '%' . $goodsName . '%');
                });
            })->where('goods_classify', 'SERVICE')
            ->where('goods_category', 'FARM')
            ->orderBy('goods_addtimes', 'DESC')
            ->forPage($page, $pageSize)
            ->get()
            ->toArray();
        $total = Goods::when($goodsName != '', function ($query) use ($goodsName) {
            $query->where(function ($q) use ($goodsName) {
                $q->where('goods_name', 'like', '%' . $goodsName . '%')
                    ->OrWhere('goods_title', 'like', '%' . $goodsName . '%');
            });
        })->where('goods_classify', 'SERVICE')
            ->where('goods_category', 'FARM')
            ->count();

        foreach ($rows as &$row) {
            $row['goods_cover'] = getenv('STORAGE_DOMAIN') . $row['goods_cover'];
            if (isset($row['running'])) {
                $row['running']['goods_running_storage'] = intval($row['running']['goods_running_storage']);
            }
        }

        return json_success('', compact('rows', 'page', 'pageSize', 'total'));
    }

    public static function selectPackage(Request $request)
    {
        $page = $request->get('page');
        $pageSize = $request->get('pageSize');
        $goodsName = $request->get('goods_name', '');
        $categoryId = $request->get('join_goods_category_id', null);

        $rows = Goods::with([
            'category' => function ($query) {
                $query->select('category_id', 'category_name');
            },
            'running' => function ($query) {
                $query->select('join_running_goods_id', 'goods_running_storage');
            },
            'supplier' => function ($query) {
                $query->select('supplier_id', 'supplier_name');
            },
        ])->select('goods_id', 'join_goods_category_id', 'join_goods_supplier_id', 'goods_status', 'goods_sales_price', 'goods_category', 'goods_name', 'goods_title', 'goods_cover', 'goods_sort', 'goods_addtimes')
            ->when($goodsName != '', function ($query) use ($goodsName) {
                $query->where(function ($q) use ($goodsName) {
                    $q->where('goods_name', 'like', '%' . $goodsName . '%')
                        ->OrWhere('goods_title', 'like', '%' . $goodsName . '%');
                });
            })->when($categoryId != null, function ($query) use ($categoryId) {
                $query->where('join_goods_category_id', $categoryId);
            })
            ->where('goods_classify', 'PACKAGE')
            ->orderBy('goods_addtimes', 'DESC')
            ->forPage($page, $pageSize)
            ->get()
            ->toArray();
        $total = Goods::when($goodsName != '', function ($query) use ($goodsName) {
            $query->where(function ($q) use ($goodsName) {
                $q->where('goods_name', 'like', '%' . $goodsName . '%')
                    ->OrWhere('goods_title', 'like', '%' . $goodsName . '%');
            });
        })->when($categoryId != null, function ($query) use ($categoryId) {
            $query->where('join_goods_category_id', $categoryId);
        })->where('goods_classify', 'PACKAGE')->count();

        foreach ($rows as &$row) {
            $row['goods_cover'] = getenv('STORAGE_DOMAIN') . $row['goods_cover'];
            if (isset($row['running'])) {
                $row['running']['goods_running_storage'] = intval($row['running']['goods_running_storage']);
            }
//            if (!empty($row['component'])) {
//                $ids = [];
//                $contentList = [];
//                foreach ($row['component'] as $component) {
//                    $ids[] = $component['join_component_goods_id'];
//                    $configJson = json_decode($component['goods_component_config_json'], true);
//                    $contentList[] = [
//                        'goods_name' => $configJson['goods_name'] ?? '',
//                        'goods_sales_price' => $component['goods_component_price'],
//                        'nbr' => $configJson['nbr'] ?? 0
//                    ];
//                }
//
//                $row['join_component_goods_id'] = $ids;
//                $row['goodsContentList'] = $contentList;
//            }
        }

        return json_success('', compact('rows', 'page', 'pageSize', 'total'));
    }

    /**
     * @Desc 下拉选择服务商品
     * @Author Gorden
     * @Date 2024/4/24 13:32
     *
     * @param Request $request
     * @return Response
     */
    public static function selectList(Request $request, $goodsClassify = "SERVICE")
    {
        $keywords = $request->get('keywords', '');
//        if (!$keywords){
//            return json_success('暂无数据');
//        }

        $categoryIds = SysCategory::whereIn('category_super_id', [5, 31, 32, 42, 66, 70, 72])->pluck('category_id');

        $goods = Goods::with('sku')
            ->whereIn('join_goods_category_id', $categoryIds)
            ->when($keywords != '', function ($query) use ($keywords) {
                $query->where('goods_name', 'like', "%" . $keywords . "%");
            })
            ->select('goods_id', 'goods_name', 'goods_sales_price', 'join_goods_category_id')
            ->get()
            ->toArray();

        foreach ($goods as &$good) {
            if (!empty($good['sku'])) {
                foreach ($good['sku'] as $key => $sku) {
                    if (!empty($sku['goods_sku_specs_json'])) {
                        $good['sku'][$key]['goods_sku_specs_json'] = json_decode($sku['goods_sku_specs_json']);

                        $skuName = '';
                        foreach ($good['sku'][$key]['goods_sku_specs_json'] as $specsKey => $skuSpecs) {
                            if (is_array($skuSpecs)) {
                                $skuName = $skuName . ' ' . $specsKey . ' ' . implode(' ', $skuSpecs);
                            } else {
                                $skuName = $skuName . ' ' . $specsKey . ' ' . $skuSpecs;
                            }
                        }
                        $good['sku'][$key]['sku_name'] = $skuName;
                    }
                }
            } else {
                $good['sku'] = [];
            }
        }

        return json_success('', $goods);

    }

    /**
     * @Desc 商品详情
     * @Author Gorden
     * @Date 2024/3/28 10:25
     *
     * @param $goodsId
     * @return Response
     */
    public static function info($goodsId)
    {
        try {
            // 商品主表
            $main = Goods::where('goods_id', $goodsId)->first();
            if (!empty($main)) {
                $main = $main->toArray();
                $main['goods_sku_json'] = json_decode($main['goods_sku_json'], true);
                $main['specList'] = [];
                foreach ($main['goods_sku_json'] as $key => $sku) {
                    $main['specList'][] = [
                        'label' => $key,
                        'tags' => $sku
                    ];
                }

            } else {
                $main = [];
            }
            // 详情表
            $detail = GoodsDetail::where('join_detail_goods_id', $goodsId)->first();
            if (!empty($detail)) {
                $detail = $detail->toArray();
            } else {
                $detail = [];
            }
            // 标签表
            $label = GoodsLabel::where('join_label_goods_id', $goodsId)->first();
            if (!empty($label)) {
                $label = $label->toArray();
            } else {
                $label = [];
            }
            // Running表
            $running = GoodsRunning::where('join_running_goods_id', $goodsId)->first();
            if (!empty($running)) {
                $running = $running->toArray();
                if (!empty($running['goods_running_off_json']) && is_json($running['goods_running_off_json'])) {
                    $goodsRunningOffJson = json_decode($running['goods_running_off_json'], true);
                    $running['goods_off_addtimes'] = isset($goodsRunningOffJson['time']) ? date("Y-m-d H:i", $goodsRunningOffJson['time']) : '';
                }
            } else {
                $running = [];
            }
            // Sku表
            $skus = GoodsSku::where('join_sku_goods_id', $goodsId)->get();
            if (!empty($skus)) {
                $skus = $skus->toArray();
                $submitList = [];
                foreach ($skus as $key => $sku) {
                    $skuSpecsJson = json_decode($sku['goods_sku_specs_json'], true);
                    $skuSpecs = '';
                    $skuNameValue = [];
                    $i = 1;
                    foreach ($skuSpecsJson as $k => $item) {
                        if (is_array($item)) {
                            $item = implode(',', $item);
                        }
                        $skuSpecs = $skuSpecs . $item . ',';
                        $skuNameKey = 'skuName' . $i;
                        $skuValueKey = 'skuValue' . $i;
                        $skuNameValue[$skuNameKey] = $k;
                        $skuNameValue[$skuValueKey] = $item;
                        $skuNameValue[$k] = $item;
                        $i++;
                    }
                    $storage = json_decode($sku['goods_sku_storage_json'], true);
                    $priceStorage = [
                        'sku_id' => $sku['goods_sku_id'],
                        'sku' => rtrim($skuSpecs, ',') ?? '',
                        'stock' => $storage['storage'] ?? '',
                        'price' => $sku['goods_sku_sales_price'] ?? '',
                    ];

                    $submitList[] = array_merge($skuNameValue, $priceStorage);
                }
                $skus['submitList'] = $submitList;
            } else {
                $skus = [];
            }

            // 合并数据
            $data = array_merge($main, $detail, $label, $running, ['sku' => $skus]);
            $data['goods_sku_json_label'] = [];

            $data['goods_label'] = !empty($data['goods_label']) ? explode(',', $data['goods_label']) : [];
            $data['goods_json'] = $data['goods_json'] ? json_decode($data['goods_json'], true) : [];
            $data['goods_cover'] = getenv('STORAGE_DOMAIN') . $data['goods_cover'];
            if (!empty($data['goods_detail_slider_json'])) {
                $data['goods_detail_slider_json'] = json_decode($data['goods_detail_slider_json'], true);
                // ……
                if (isset($data['goods_detail_slider_json']['slider'])) {
                    $data['goods_detail_slider_json'] = explode(',', $data['goods_detail_slider_json']['slider']);
                }

                $slider = '';
                foreach ($data['goods_detail_slider_json'] as $item) {
                    $slider .= getenv('STORAGE_DOMAIN') . $item . ',';
                }
                $data['goods_detail_slider_json'] = rtrim($slider, ',');
            }
            $extendJson = [];
            if (!empty($data['goods_attribute_json'])) {
                $extendJson = json_decode($data['goods_attribute_json'], true);
            }
            $data['appointment_times'] = [];
            if ($data['is_support_appointment'] == 'Y' && isset($extendJson['dates'])) {
                $data['dates'] = $extendJson['dates'] ?? [];
                if (isset($extendJson['times'])) {
                    $times = [];
                    foreach ($extendJson['times'] as $time) {
                        if (!empty($time['duration'])) {
                            $startEndTime = explode('-', $time['duration']);
                            $times[] = [
                                'person' => $time['person'],
                                'appointmentTimeStart' => $startEndTime[0],
                                'appointmentTimeEnd' => $startEndTime[1],
                            ];
                        }
                    }

                    $data['appointment_times'] = $times;
                }
                if (isset($extendJson['address'])) {
                    $data['address'] = $extendJson['address'];
                }
                if (isset($extendJson['position'])) {
                    $data['position'] = $extendJson['position'];
                }
                if (isset($extendJson['label'])) {
                    $data['appointment_label'] = $extendJson['label'];
                }
            }
            if (!empty($data['goods_json']) && $data['join_goods_category_id'] == 65) {
                foreach ($data['goods_json'] as $key => $datum) {
                    $data['goods_json'][$key]['color'] = rgbToHex($datum['color']);
                }
            } elseif (!empty($data['goods_json']) && $data['join_goods_category_id'] == 43) {
//                dump($data['goods_json']);
                foreach ($data['goods_json'] as $key1 => $item1) {
                    $data['goods_json'][$key1]['title'] = $key1;
                    foreach ($item1['items'] as $key2 => $item2) {
                        $data['goods_json'][$key1]['items'][$key2]['key'] = $key2;
                        $data['goods_json'][$key1]['items'][$key2]['params'] = $item2;
                    }
                }
            }

            $data['goods_on_addtimes'] = date('Y-m-d\TH:i:s.u\Z', $data['goods_on_addtimes'] - 60 * 60 * 8);

            return json_success('', $data);
        } catch (\Exception $e) {
            dump($e->getMessage());
            return json_fail("查询错误~");
        }
    }

    public static function infoPackage($goodsId)
    {
        try {
            // 商品主表
            $main = Goods::where('goods_id', $goodsId)->first();
            if (!empty($main)) {
                $main = $main->toArray();
                $main['goods_sku_json'] = json_decode($main['goods_sku_json'], true);
                $main['specList'] = [];
                foreach ($main['goods_sku_json'] as $key => $sku) {
                    $main['specList'][] = [
                        'label' => $key,
                        'tags' => $sku
                    ];
                }

            } else {
                $main = [];
            }
            // 详情表
            $detail = GoodsDetail::where('join_detail_goods_id', $goodsId)->first();
            if (!empty($detail)) {
                $detail = $detail->toArray();
            } else {
                $detail = [];
            }
            // 标签表
            $label = GoodsLabel::where('join_label_goods_id', $goodsId)->first();
            if (!empty($label)) {
                $label = $label->toArray();
            } else {
                $label = [];
            }
            // Running表
            $running = GoodsRunning::where('join_running_goods_id', $goodsId)->first();
            if (!empty($running)) {
                $running = $running->toArray();
                if (!empty($running['goods_running_off_json']) && is_json($running['goods_running_off_json'])) {
                    $goodsRunningOffJson = json_decode($running['goods_running_off_json'], true);
                    $running['goods_off_addtimes'] = isset($goodsRunningOffJson['time']) ? date("Y-m-d H:i", $goodsRunningOffJson['time']) : '';
                }
            } else {
                $running = [];
            }
            // 组件表
            $component = GoodsComponent::where('join_component_master_goods_id', $goodsId)->get()->toArray();
            $componentArr['join_component_goods_id'] = [];
            $componentArr['goodsContentList'] = [];
            if ($component) {
                $ids = [];
                $contentList = [];
                foreach ($component as $item) {
                    if (empty($item['join_component_goods_id'])) {
                        continue;
                    }
                    $skus = GoodsSku::where('join_sku_goods_id', $item['join_component_goods_id'])
                        ->select('goods_sku_id', 'join_sku_goods_id', 'goods_sku_specs_json', 'goods_sku_sales_price')
                        ->get()
                        ->toArray();
                    foreach ($skus as $key => $sku2) {
                        if (!empty($sku2['goods_sku_specs_json'])) {
                            $skus[$key]['goods_sku_specs_json'] = json_decode($sku2['goods_sku_specs_json'], true);

                            $skuName = '';
                            foreach ($skus[$key]['goods_sku_specs_json'] as $specsKey => $skuSpecs) {
                                if (is_array($skuSpecs)) {
                                    $skuName = $skuName . ' ' . $specsKey . ' ' . implode(' ', $skuSpecs);
                                } else {
                                    $skuName = $skuName . ' ' . $specsKey . ' ' . $skuSpecs;
                                }
                            }
                            $skus[$key]['sku_name'] = $skuName;
                        }
                    }
                    $ids[] = $item['join_component_goods_id'];
                    $configJson = json_decode($item['goods_component_json'], true);
                    $contentList[] = [
                        'goods_id' => $item['join_component_goods_id'],
                        'goods_name' => $configJson['goods_name'] ?? '',
                        'goods_sales_price' => $item['goods_component_price'],
                        'nbr' => $configJson['nbr'] ?? 0,
                        'sku_id' => $configJson['sku_id'] ?? '',
                        'skus' => $skus
                    ];
                }

                $componentArr['join_component_goods_id'] = $ids;
                $componentArr['goodsContentList'] = $contentList;
            }

            // 合并数据
            $data = array_merge($main, $detail, $label, $running, $componentArr);
            $data['goods_sku_json_label'] = [];

            $data['goods_label'] = !empty($data['goods_label']) ? explode(',', $data['goods_label']) : [];
            $data['goods_cover'] = getenv('STORAGE_DOMAIN') . $data['goods_cover'];
            if (!empty($data['goods_detail_slider_json'])) {
                $data['goods_detail_slider_json'] = json_decode($data['goods_detail_slider_json'], true);
                $slider = '';
                foreach ($data['goods_detail_slider_json'] as $item) {
                    $slider .= getenv('STORAGE_DOMAIN') . $item . ',';
                }
                $data['goods_detail_slider_json'] = rtrim($slider, ',');
            }

            $data['goods_on_addtimes'] = date('Y-m-d\TH:i:s.u\Z', $data['goods_on_addtimes'] - 60 * 60 * 8);

            return json_success('', $data);
        } catch (\Exception $e) {
            dump($e->getTrace());
            return json_fail("查询错误~");
        }
    }


    /**
     * @Desc 添加商品
     * @Author Gorden
     * @Date 2024/3/11 10:20
     *
     * @param $params
     * @return Response
     */
    public static function insert($params): Response
    {
        Db::beginTransaction();
        try {
            $params['goods_id'] = "GD" . sprintf('%016d', SysSerial::getSerial()) . random_string(6, 'up');
            // 主表
            self::mainInsert($params);
            // 商品详情表
            self::detailInsert($params);
            // 商品标签表
            self::labelInsert($params);
            // 产品运行控制信息表
            self::goodsRunningInsert($params);
            // sku表
            self::goodsSkuSet($params, 'insert');
            // 待上架状态，上架时间大于当前时间
            if ($params['goods_status'] == 'PENDING' && strtotime($params['goods_on_addtimes']) > time()) {
                $redis = Redis::connection();
                $key = date('YmdHi', strtotime($params['goods_on_addtimes']));
                $redis->sAdd(Goods::LISTING_KEY_PREFIX . $key, $params['goods_id']);
            }
            // 自动下架
            if (!empty($params['goods_running_off_type']) && $params['goods_running_off_type'] == 'T' && !empty($params['goods_off_addtimes'])) {
                $redis = Redis::connection();
                $key = Goods::LISTING_OFF_KEY_PREFIX . date('YmdHi', strtotime($params['goods_off_addtimes']));
                $redis->sAdd($key, $params['goods_id']);
            }
            Db::commit();
        } catch (\PDOException $e) {
            Db::rollBack();
            dump($e->getMessage());
            return json_fail('数据写入失败~');
        } catch (BusinessException $e) {
            Db::rollBack();
            dump($e->getMessage());
            return json_fail($e->getMessage());
        } catch (\Exception $e) {
            Db::rollBack();
            dump($e->getTrace());
            return json_fail('数据写入失败~');
        }

        _syslog("添加商品", "商品名【" . $params['goods_name'] . "】");

        return json_success('success');
    }

    public static function insertPackage($params): Response
    {
        Db::beginTransaction();
        try {
            $params['goods_id'] = "GD" . sprintf('%016d', SysSerial::getSerial()) . random_string(6, 'up');
            // 主表
            self::mainInsert($params);
            // 商品详情表
            self::detailInsert($params);
            // 套包组件表
            self::componentUpdate($params, 'insert');
            // 商品标签表
            self::labelInsert($params);
            // 产品运行控制信息表
            self::goodsRunningInsert($params);
            // sku表
            self::goodsSkuSet($params, 'insert');
            // 待上架状态，上架时间大于当前时间
            if ($params['goods_status'] == 'PENDING' && strtotime($params['goods_on_addtimes']) > time()) {
                $redis = Redis::connection();
                $key = date('YmdHi', strtotime($params['goods_on_addtimes']));
                $redis->sAdd(Goods::LISTING_KEY_PREFIX . $key, $params['goods_id']);
            }
            Db::commit();
        } catch (\PDOException $e) {
            Db::rollBack();
            dump($e->getMessage());
            return json_fail('数据写入失败~');
        } catch (BusinessException $e) {
            Db::rollBack();
            dump($e->getMessage());
            return json_fail($e->getMessage());
        } catch (\Exception $e) {
            Db::rollBack();
            dump($e->getTrace());
            return json_fail('数据写入失败~');
        }

        _syslog("添加套餐", "商品名【" . $params['goods_name'] . "】");

        return json_success('success');
    }

    public static function update($params)
    {
        Db::beginTransaction();
        try {
            // 主表
            self::mainUpdate($params);
            // 商品详情表
            self::detailUpdate($params);
            // 商品标签表
            self::labelUpdate($params);
            // 产品运行控制信息表
            self::goodsRunningUpdate($params);
            // sku表
            self::goodsSkuSet($params, 'update');

            Db::commit();
        } catch (BusinessException $e) {
            Db::rollBack();
            return json_fail($e->getMessage());
        } catch (\Exception $e) {
            Db::rollBack();
            return json_fail('数据更新失败~');
        }

        _syslog("编辑商品", "商品名【" . $params['goods_name'] . "】" ?? "商品ID:【" . $params['goods_id'] . "】");

        return json_success('success');
    }

    public static function changeStatus($params)
    {
        try {
            Goods::where('goods_id', $params['goods_id'])->update(['goods_status' => $params['goods_status']]);

            return json_success('修改成功');
        } catch (\Exception $e) {
            return json_fail('修改状态失败');
        }
    }


    public static function updatePackage($params)
    {
        Db::beginTransaction();
        try {
            // 主表
            self::mainUpdate($params);
            // 商品详情表
            self::detailUpdate($params);
            // 套包组件表
            self::componentUpdate($params, 'update');
            // 商品标签表
            self::labelUpdate($params);
            // 产品运行控制信息表
            self::goodsRunningUpdate($params);

            Db::commit();
        } catch (BusinessException $e) {
            Db::rollBack();
            return json_fail($e->getMessage());
        } catch (\Exception $e) {
            Db::rollBack();
            dump($e->getTrace());
            return json_fail('数据更新失败~');
        }
        _syslog("编辑套餐", "商品名【" . $params['goods_name'] . "】" ?? "商品ID:【" . $params['goods_id'] . "】");

        return json_success('success');
    }

    /**
     * @Desc 删除商品
     * @Author Gorden
     * @Date 2024/3/28 13:20
     *
     * @param $ids
     * @return Response
     */
    public static function delete($ids)
    {
        if (!$ids) {
            return json_fail("数据错误~");
        }
        if (!is_array($ids)) {
            $ids = [$ids];
        }

        $goods = Goods::whereIn('goods_id', $ids)->get()->toArray();
        if (!$goods) {
            return json_fail("数据错误~");
        }

        Db::beginTransaction();
        try {
            Goods::whereIn('goods_id', $ids)->delete();
            GoodsDetail::whereIn('join_detail_goods_id', $ids)->delete();
            GoodsLabel::whereIn('join_label_goods_id', $ids)->delete();
            GoodsRunning::whereIn('join_running_goods_id', $ids)->delete();
            GoodsSku::whereIn('join_sku_goods_id', $ids)->delete();

            Db::commit();

            _syslog("删除商品 / 套餐", "ID:【" . implode(',', $ids) . "】", $goods);

            return json_success("商品删除成功");
        } catch (\Exception $e) {
            Db::rollBack();

            return json_fail("商品删除失败~");
        }
    }

    /**
     * @Desc 商品主表
     * @Author Gorden
     * @Date 2024/3/11 11:20
     *
     * @param $params
     * @return mixed|string
     * @throws BusinessException
     */
    public static function mainInsert($params)
    {
        if (!empty($params['goods_cover'])) {
            $params['goods_cover'] = str_replace(getenv('STORAGE_DOMAIN'), '', $params['goods_cover']);
        }
        // 如果产品是待处理状态
        if ($params['goods_status'] == 'PENDING') {
            if (strtotime($params['goods_on_addtimes']) <= time()) {
                $params['goods_status'] = 'ON';
            }
        }
        $category = SysCategory::where('category_id', $params['join_goods_category_id'])->first();
        if (!$category) {
            throw new BusinessException("产品分类不存在~");
        }

        try {
            $model = new Goods();
            $model->goods_id = $params['goods_id'];
            $model->join_goods_category_id = $params['join_goods_category_id'] ?? 0;
            $model->join_goods_supplier_id = $params['join_goods_supplier_id'] ?? 0;
            $model->goods_classify = $params['goods_classify'] ?? '';
            $model->goods_status = $params['goods_status'] ?? '';
            $model->goods_category = $category->category_classify ?? '';
            $model->goods_prefix = $params['goods_prefix'] ?? ($category->category_name ? '【' . $category->category_name . '】' : '');
            $model->goods_name = $params['goods_name'];
            $model->goods_market_price = $params['goods_market_price'] ?? 0;
            $model->goods_sales_price = $params['goods_sales_price'] ?? 0;
            $model->goods_sku_json = !empty($params['goods_sku_json_label']) ? json_encode($params['goods_sku_json_label']) : json_encode(['规格' => []]);
            $model->goods_attribute_json = !empty($params['goods_attribute_json']) ? $params['goods_attribute_json'] : '{}';
//            $model->goods_service_json = !empty($params['goods_service_json']) ? $params['goods_service_json'] : '{}';
            $model->goods_title = $params['goods_title'] ?? '';
            $model->goods_cover = $params['goods_cover'] ?? '';
            $model->goods_on_addtimes = $params['goods_on_addtimes'] ? strtotime($params['goods_on_addtimes']) : null;
            $model->goods_sort = $params['goods_sort'] ?? null;
            $model->goods_groupby = $params['goods_groupby'] ?? '';
            $model->goods_remark = $params['goods_remark'] ?? '';
            $model->goods_extend_json = $params['goods_extend_json'] ?? '{}';
            $model->is_support_appointment = $params['is_support_appointment'] ?? 'N';
            $model->goods_addtimes = time();

            if (!empty($params['is_support_appointment']) && $params['is_support_appointment'] == 'Y' && !empty($params['appointment_times'])) {
                $times = [];
                foreach ($params['appointment_times'] as $time) {
                    $times[$time['appointmentTimeStart']] = [
                        'person' => $time['person'],
                        'duration' => $time['appointmentTimeStart'] . '-' . $time['appointmentTimeEnd']
                    ];
                }
                $model->goods_attribute_json = json_encode([
                    'icon' => '',
                    'dates' => $params['dates'] ?? [],
                    'times' => $times
                ]);
            }
            if (!empty($params['goods_json']) && $params['join_goods_category_id'] == 65) {
                $goodsJson = json_decode($params['goods_json'], true);
                foreach ($goodsJson as $key => $item) {
                    $goodsJson[$key]['color'] = hexToRgb($item['color']);
                }

                $model->goods_json = json_encode($goodsJson);
            } elseif (!empty($params['goods_json']) && $params['join_goods_category_id'] == 43) {
                $goodsJson = json_decode($params['goods_json'], true);
                $newGoodsJson = [];
                foreach ($goodsJson as $item1) {
                    $newItem1 = [];
                    foreach ($item1['items'] as $item2) {
                        $newItem1['items'][$item2['key']] = $item2['params'];
                    }
                    $newItem1['service'] = $item1['service'];
                    $newGoodsJson[$item1['title']] = $newItem1;
                }
                $model->goods_json = json_encode($newGoodsJson);
            } else {
                $model->goods_json = '[]';
            }

            if ($model->save()) {
                return $model->goods_id;
            }
            // 异常
            throw new BusinessException("数据写入失败~");
        } catch (\Exception $e) {
            dump($e->getMessage());
            throw new BusinessException("数据写入失败~");
        }
    }

    /**
     * @Desc 详情表
     * @Author Gorden
     * @Date 2024/3/11 11:19
     *
     * @param $params
     * @return void
     * @throws BusinessException
     */
    public static function detailInsert($params)
    {
        if (!empty($params['goods_detail_slider_json'])) {
            $params['goods_detail_slider_json'] = str_replace(getenv('STORAGE_DOMAIN'), '', $params['goods_detail_slider_json']);
            $params['goods_detail_slider_json'] = json_encode(['slider' => $params['goods_detail_slider_json']]);
        }
        try {
            $model = new GoodsDetail();
            $model->join_detail_goods_id = $params['goods_id'];
            $model->goods_detail_code_json = $params['goods_detail_code_json'] ?? '{}';
            $model->goods_detail_slider_json = $params['goods_detail_slider_json'] ?? '{}';
            $model->goods_detail_specs_json = $params['goods_detail_specs_json'] ?? '{}';
            $model->goods_detail_content = $params['goods_detail_content'] ?? '';
            if (!$model->save()) {
                // 异常
                throw new BusinessException("数据写入失败~");
            }
        } catch (\Exception $e) {
            dump($e->getMessage());
            throw new BusinessException("数据写入失败~");
        }
    }

    public static function componentUpdate($params, $type = 'insert')
    {
        Db::beginTransaction();
        try {
            // 有先删除
            if ($type == 'update') {
                GoodsComponent::where('join_component_master_goods_id', $params['goods_id'])->delete();
            } else {
                Goods::where('goods_id', $params['goods_id'])->update(['goods_sku_json' => '{"规格": ["标准"]}']);
                $skuData = [
                    'join_sku_goods_id' => $params['goods_id'],
                    'goods_sku_status' => 'ON',
                    'goods_sku_specs_json' => '{"规格": "标准"}',
                    'goods_sku_title' => "标准" . $params['goods_name'],
                    'goods_sku_market_price' => $params['goods_market_price'] ?? 0,
                    'goods_sku_sales_price' => $params['goods_sales_price'] ?? 0,
                ];
                GoodsSku::insert($skuData);
            }
            $data = [];
            foreach ($params['goods_content_list'] as $item) {
                $goods = Goods::where('goods_id', $params['goods_id'])->first();
                if (!$goods) {
                    continue;
                }

                $data[] = [
                    'join_component_master_goods_id' => $params['goods_id'],
                    'join_component_goods_id' => $item['goods_id'] ?? '',
                    'goods_component_price' => $item['goods_sales_price'] ?? '',
                    'goods_component_cover' => $goods->goods_cover,
                    'goods_component_json' => json_encode(['goods_name' => $item['goods_name'], 'nbr' => $item['nbr'], 'sku_id' => $item['sku_id'] ?? '']),
                    'goods_component_addtimes' => time()
                ];
            }
            dump($data);

            if ($data) {
                GoodsComponent::insert($data);
            }
            Db::commit();
        } catch (\Exception $e) {
            Db::rollBack();
            dump($e->getMessage());
            throw new BusinessException("数据写入失败~");
        }
    }

    /**
     * @Desc 标签表
     * @Author Gorden
     * @Date 2024/3/11 11:32
     *
     * @param $params
     * @return void
     * @throws BusinessException
     */
    public static function labelInsert($params)
    {
        $model = new GoodsLabel();
        $model->join_label_goods_id = $params['goods_id'];
        $model->goods_label = $params['goods_label'] ? implode(',', $params['goods_label']) : '';
        $model->goods_label_extend_json = !empty($params['goods_label_extend_json']) ? $params['goods_label_extend_json'] : '{}';
        if (!$model->save()) {
            // 异常
            throw new BusinessException('数据写入失败~');
        }
    }

    /**
     * @Desc 产品运行控制信息表
     * @Author Gorden
     * @Date 2024/3/11 11:38
     *
     * @param $params
     * @return void
     * @throws BusinessException
     */
    public static function goodsRunningInsert($params)
    {
        try {
            $model = new GoodsRunning();
            $model->join_running_goods_id = $params['goods_id'];
            $model->goods_running_storage = $params['goods_running_storage'] ?? '';
            $model->goods_running_sale = $params['goods_running_sale'] ?? '';
            $model->goods_running_off_type = !empty($params['goods_running_off_type']) ? $params['goods_running_off_type'] : '';
            $model->goods_running_off_json = !empty($params['goods_running_off_type']) && $params['goods_running_off_type'] == 'T' && !empty($params['goods_off_addtimes']) ? json_encode(['time' => strtotime($params['goods_off_addtimes'])]) : '[]';
            if (!$model->save()) {
                throw new BusinessException('数据写入失败');
            }
        } catch (\Exception $e) {
            dump($e->getMessage());
            throw new BusinessException('数据写入失败');
        }
    }

    /**
     * @Desc SKU
     * @Author Gorden
     * @Date 2024/3/11 12:01
     *
     * @param $params
     * @return void
     * @throws BusinessException
     */
    public static function skuInsert($params)
    {
        $model = new GoodsSku();
        $model->join_sku_goods_id = $params['goods_id'];
        $model->goods_sku_status = $params['goods_sku_status'];
        $model->goods_sku_specs_json = $params['goods_sku_specs_json'];
        $model->goods_sku_title = $params['goods_sku_title'];
        $model->goods_sku_images_json = $params['goods_sku_images_json'];
        $model->goods_sku_content = $params['goods_sku_content'];
        $model->goods_sku_market_price = $params['goods_sku_market_price'];
        $model->goods_sku_sales_price = $params['goods_sku_sales_price'];
        $model->goods_sku_storage_json = $params['goods_sku_storage_json'];
        $model->goods_sku_service_json = $params['goods_sku_service_json'];
        $model->goods_sku_extend_json = $params['goods_sku_extend_json'];
        if (!$model->save()) {
            throw new BusinessException('数据写入失败~');
        }
    }

    /**
     * @Desc
     * @Author Gorden
     * @Date 2024/3/12 8:44
     *
     * @param $params
     * @return void
     * @throws BusinessException
     */
    public static function mainUpdate($params)
    {
        try {
            $data = self::inputFilter($params, new Goods());
            if (!empty($data['goods_cover'])) {
                $data['goods_cover'] = str_replace(getenv('STORAGE_DOMAIN'), '', $data['goods_cover']);
            }
            $data['goods_on_addtimes'] = strtotime($data['goods_on_addtimes']);
            $data['goods_sku_json'] = !empty($params['goods_sku_json_label']) ? json_encode($params['goods_sku_json_label']) : json_encode(['规格' => []]);

            $row = Goods::find($data['goods_id']);
            if ($row->join_goods_category_id != $data['join_goods_category_id']) {
                $category = SysCategory::where('category_id', $params['join_goods_category_id'])->first();
                if (!$category) {
                    throw new BusinessException("产品分类不存在~");
                }
                $data['goods_category'] = $category->category_classify ?? '';
                $data['goods_prefix'] = $data['goods_prefix'] ?? ($category->category_name ? '【' . $category->category_name . '】' : '');
            }
            // 上架时间有变动
            if ($data['goods_status'] == 'PENDING' && $row->goods_on_addtimes != $data['goods_on_addtimes']) {
                $redis = Redis::connection();
                // 删掉原来的
                $oldKey = Goods::LISTING_KEY_PREFIX . date('YmdHi', $row->goods_on_addtimes);
                $redis->srem($oldKey, $data['goods_id']);

                // 加入新的
                $newKey = Goods::LISTING_KEY_PREFIX . date('YmdHi', $data['goods_on_addtimes']);
                $redis->sadd($newKey, $data['goods_id']);
            }
            if (!empty($params['is_support_appointment']) && $params['is_support_appointment'] == 'Y' && !empty($params['appointment_times'])) {
                $times = [];
                foreach ($params['appointment_times'] as $time) {
                    $times[$time['appointmentTimeStart']] = [
                        'person' => $time['person'],
                        'duration' => $time['appointmentTimeStart'] . '-' . $time['appointmentTimeEnd']
                    ];
                }
                $attributeJson = [
                    'icon' => '',
                    'dates' => $params['dates'] ?? [],
                    'times' => $times
                ];
                if (!empty($params['appointment_label'])) {
                    $attributeJson['label'] = $params['appointment_label'];
                }
                if (!empty($params['address'])) {
                    $attributeJson['address'] = $params['address'];
                    $attributeJson['position'] = $params['position'];
                }
                $data['goods_attribute_json'] = json_encode($attributeJson);
            }
            if (!empty($data['goods_json']) && $data['join_goods_category_id'] == 65) {
                $goodsJson = json_decode($data['goods_json'], true);
                foreach ($goodsJson as $key => $item) {
                    $goodsJson[$key]['color'] = hexToRgb($item['color']);
                }

                $data['goods_json'] = json_encode($goodsJson);
            } elseif (!empty($data['goods_json']) && $data['join_goods_category_id'] == 43) {
                $goodsJson = json_decode($data['goods_json'], true);
                $newGoodsJson = [];
                foreach ($goodsJson as $item1) {
                    $newItem1 = [];
                    foreach ($item1['items'] as $item2) {
                        $newItem1['items'][$item2['key']] = $item2['params'];
                    }
                    $newItem1['service'] = $item1['service'];
                    $newGoodsJson[$item1['title']] = $newItem1;
                }
                $data['goods_json'] = json_encode($newGoodsJson);
            } else {
                $data['goods_json'] = '[]';
            }

            dump($data);

            foreach ($data as $key => $val) {
                $row->{$key} = $val;
            }
            $row->save();
        } catch (BusinessException $e) {
            throw new BusinessException($e->getMessage());
        } catch (\Exception $e) {
            dump($e->getMessage());
            throw new BusinessException('数据更新异常~1');
        }
    }

    /**
     * @Desc
     * @Author Gorden
     * @Date 2024/3/12 9:57
     *
     * @param $params
     * @return void
     * @throws BusinessException
     */
    public static function detailUpdate($params)
    {
        try {
            $data = self::inputFilter($params, new GoodsDetail());
            if (!empty($data['goods_detail_slider_json'])) {
                $data['goods_detail_slider_json'] = str_replace(getenv('STORAGE_DOMAIN'), '', $data['goods_detail_slider_json']);
                $data['goods_detail_slider_json'] = json_encode(['slider' => $data['goods_detail_slider_json']]);
            }
            // 根据goods_id 查详情ID
            $detail = GoodsDetail::where('join_detail_goods_id', $params['goods_id'])->first();
            if ($detail) {
                self::doUpdate($detail->join_detail_goods_id, $data, new GoodsDetail());
            } else {
                $data['join_detail_goods_id'] = $params['goods_id'];
                GoodsDetail::insert($data);
            }
        } catch (BusinessException $e) {
            throw new BusinessException($e->getMessage());
        } catch (\Exception $e) {
            dump($e->getMessage());
            throw new BusinessException('数据更新异常~2');
        }
    }

    /**
     * @Desc
     * @Author Gorden
     * @Date 2024/3/12 9:58
     *
     * @param $params
     * @return void
     * @throws BusinessException
     */
    public static function labelUpdate($params)
    {
        try {
            $data = self::inputFilter($params, new GoodsLabel());
            dump($data);
            // 根据goods_id 查详情ID
            $detail = GoodsLabel::where('join_label_goods_id', $params['goods_id'])->first();
            if ($detail) {
                self::doUpdate($detail->goods_label_id, $data, new GoodsLabel());
            }else{
                $data['join_label_goods_id'] = $params['goods_id'];
                GoodsLabel::insert($data);
            }
        } catch (BusinessException $e) {
            throw new BusinessException($e->getMessage());
        } catch (\Exception $e) {
            dump($e->getMessage());
            throw new BusinessException('数据更新异常~3');
        }
    }

    /**
     * @Desc
     * @Author Gorden
     * @Date 2024/3/12 9:59
     *
     * @param $params
     * @return void
     * @throws BusinessException
     */
    public static function goodsRunningUpdate($params)
    {
        try {
            $data = self::inputFilter($params, new GoodsRunning());
            // 根据goods_id 查详情ID
            $detail = GoodsRunning::where('join_running_goods_id', $params['goods_id'])->first();
            if (!empty($params['goods_running_off_type']) && $params['goods_running_off_type'] == 'T') {
                $redis = Redis::connection();
                if (!empty($detail->goods_running_off_json)) {
                    $goodsRunningOffJson = json_decode($detail->goods_running_off_json, true);
                    if (isset($goodsRunningOffJson['time'])) {
                        $oldKey = Goods::LISTING_OFF_KEY_PREFIX . date('YmdHi', $goodsRunningOffJson['time']);
                        $goodsRunningOffJson['time'] = strtotime($params['goods_off_addtimes']);
                        $data['goods_running_off_json'] = json_encode($goodsRunningOffJson);
                        // 有老的下架时间，删除老的
                        $redis->srem($oldKey, $params['goods_id']);
                    } else {
                        $goodsRunningOffJson['time'] = strtotime($params['goods_off_addtimes']);
                        $data['goods_running_off_json'] = json_encode($goodsRunningOffJson);
                    }
                } else {
                    $data['goods_running_off_json'] = json_encode(['time' => strtotime($params['goods_off_addtimes'])]);
                }
                // 加入自动下架
                $newKey = Goods::LISTING_OFF_KEY_PREFIX . date('YmdHi', strtotime($params['goods_off_addtimes']));
                $redis->sAdd($newKey, $params['goods_id']);

            } else if (!empty($params['goods_running_off_type']) && !empty($detail->goods_running_off_type) && $params['goods_running_off_type'] == 'H' && $detail->goods_running_off_type == 'T') {
                $goodsRunningOffJson = json_decode($detail->goods_running_off_json, true);
                if (isset($goodsRunningOffJson['time'])) {
                    $oldKey = Goods::LISTING_OFF_KEY_PREFIX . date('YmdHi', $goodsRunningOffJson['time']);
                    $redis = Redis::connection();
                    $redis->srem($oldKey, $params['goods_id']);
                }
            }
            if ($detail) {
                self::doUpdate($detail->join_running_goods_id, $data, new GoodsRunning());
            } else {
                // 兼容老数据……
                $data['join_running_goods_id'] = $params['goods_id'];
                GoodsRunning::insert($data);
            }

        } catch (BusinessException $e) {
            throw new BusinessException($e->getMessage());
        } catch (\Exception $e) {
            dump($e->getMessage());
            throw new BusinessException('数据更新异常~4');
        }
    }

    /**
     * @Desc sku 设置
     * @Author Gorden
     * @Date 2024/4/10 10:43
     *
     * @param $params
     * @return void
     * @throws BusinessException
     */
    public static function goodsSkuSet($params, $operation = 'insert')
    {
        try {
            Db::beginTransaction();
            $skusOldIds = [];
            if ($operation == 'update') {
                // 查出所有的
                $skusOldIds = GoodsSku::where('join_sku_goods_id', $params['goods_id'])->pluck('goods_sku_id', 'goods_sku_id');
                // 删掉原有的
//                GoodsSku::where('join_sku_goods_id', $params['goods_id'])->delete();
            }
            // 入新的
            if (!empty($params['goods_sku_json_value'])) {
                foreach ($params['goods_sku_json_value'] as $item) {
                    $skus = explode(',', $item['sku']);
                    $skuArr = [];
                    for ($i = 1; $i <= count($skus); $i++) {
                        $skuName = "skuName" . $i;
                        $key = $item[$skuName];
                        $skuArr[$key] = $skus[$i - 1];
                    }
                    $specsJson = json_encode($skuArr);
                    $skuTitle = str_replace('-', ',', $item['sku']) . $params['goods_name'];
                    if ($operation == 'update' && !empty($item['sku_id'])) {
                        $model = GoodsSku::where('goods_sku_id', $item['sku_id'])->first();
                        if (!$model) {
                            $model = new GoodsSku();
                        } else {
                            unset($skusOldIds[$model->goods_sku_id]);
                        }
                    } else {
                        $model = GoodsSku::where('join_sku_goods_id', $params['goods_id'])->where('goods_sku_title', $skuTitle)->first();
                        if (!$model) {
                            $model = new GoodsSku();
                        } else {
                            dump($model->goods_sku_id);
                            unset($skusOldIds[$model->goods_sku_id]);
                        }

                    }

                    $model->join_sku_goods_id = $params['goods_id'];
                    $model->goods_sku_status = $params['goods_status'];
                    $model->goods_sku_specs_json = $specsJson;
                    $model->goods_sku_title = $skuTitle;
                    $model->goods_sku_market_price = $params['goods_market_price'] ?? 0;
                    $model->goods_sku_sales_price = $item['price'];
                    $model->goods_sku_storage_json = json_encode(['storage' => $item['stock']]);
                    $model->save();

                }
            }
            if ($operation == 'update' && !empty($skusOldIds)) {
                GoodsSku::whereIn('goods_sku_id', $skusOldIds)->delete();
            }
            Db::commit();
        } catch (\Exception $e) {
            dump($e->getTrace());
            Db::rollBack();

            throw new BusinessException('数据更新异常~5');
        }
    }

    /**
     * @Desc
     * @Author Gorden
     * @Date 2024/3/12 8:45
     *
     * @param array $data
     * @param $model
     * @return array
     * @throws BusinessException
     */
    private static function inputFilter(array $data, $model): array
    {
        $table = config('database.connections.mysql.prefix') . $model->getTable();
        $allow_column = $model->getConnection()->select("desc `$table`");
        if (!$allow_column) {
            throw new BusinessException('表不存在', 2);
        }
        $columns = array_column($allow_column, 'Type', 'Field');
        foreach ($data as $col => $item) {
            if (!isset($columns[$col])) {
                unset($data[$col]);
                continue;
            }
            // 非字符串类型传空则为null
            if ($item === '' && strpos(strtolower($columns[$col]), 'varchar') === false && strpos(strtolower($columns[$col]), 'text') === false) {
                $data[$col] = null;
            }
            if (is_array($item)) {
                $data[$col] = implode(',', $item);
            }
            if ($item != '' && (strpos(strtolower($columns[$col]), 'varchar') || strpos(strtolower($columns[$col]), 'text'))) {
                $data[$col] = htmlspecialchars($item);
            }
        }
        if (empty($data['created_at'])) {
            unset($data['created_at']);
        }
        if (empty($data['updated_at'])) {
            unset($data['updated_at']);
        }
        return $data;
    }

    /**
     * @Desc 执行更新
     * @Author Gorden
     * @Date 2024/3/12 8:43
     *
     * @param $id
     * @param $data
     * @param $model
     * @return void
     */
    private static function doUpdate($id, $data, $model)
    {
        $row = $model->find($id);
        foreach ($data as $key => $val) {
            $row->{$key} = $val;
        }
        $row->save();
    }

    /**
     * @Desc 上架定时
     * @Author Gorden
     * @Date 2024/4/11 15:13
     *
     * @return void
     */
    public static function checkListing()
    {
        $key = Goods::LISTING_KEY_PREFIX . date('YmdHi');
        $redis = Redis::connection();
        if (!$redis->exists($key)) {
            return;
        }

        $goodsIds = $redis->sMembers($key);
        if (Goods::whereIn('goods_id', $goodsIds)->update(['goods_status' => 'ON'])) {
            $redis->del($key);
        }
    }

    /**
     * @Desc 自动下架
     * @Author Gorden
     * @Date 2024/4/26 15:26
     *
     * @return void
     */
    public static function checkOffListing()
    {
        $key = Goods::LISTING_OFF_KEY_PREFIX . date('YmdHi');
        $redis = Redis::connection();
        if (!$redis->exists($key)) {
            return;
        }

        $goodsIds = $redis->sMembers($key);
        if (Goods::whereIn('goods_id', $goodsIds)->update(['goods_status' => 'OFF'])) {
            $redis->del($key);
        }
    }
}